/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package ar

import (
	"internal/cfmt"

	"fmt"
	"sort"
	"time"
	)

//-----------------------------------------------------------------------------
// Get a slice of subtask descriptions of a task with a given id

func (a *Archive) getTaskDesc(task_id TaskId, format cfmt.CFormat) (datetime time.Time,
		desc []string, err error) {

	task, ok := a.tasks[task_id]
	if !ok {
		err = fmt.Errorf("Wrong task id %d", task_id)
		return
		}
	if len(task) == 0 {
		err = fmt.Errorf("Task %s is empty", task_id)
		return
		}

	// Forming a sorted slice of subtask ids
	subtask_ids := make([]SubTaskId, 0, len(task))
	for subtask_id := range task {
		subtask_ids = append(subtask_ids, subtask_id)
		}
	sort.Slice(subtask_ids, func(i, j int) bool {
		return subtask_ids[i] < subtask_ids[j]
		})

	// For every subtask id
	for _, subtask_id := range subtask_ids {

		// Getting a subtask desc, a src name and a timestamp
		subtask, ok := task[subtask_id]
		if !ok {
			err = fmt.Errorf("Can't find subtask %d", subtask_id)
			return
			}
		src_name, ok := a.packages[subtask.Src.ID]
		if !ok {
			err = fmt.Errorf("Can't find package name of subtask %d",
				subtask_id)
			return
			}
		if len(desc) == 0 {
			datetime = time.Unix(int64(subtask.TS), 0)
			}

		// Using standard format or a custom one
		var entry string
		if format.Size() == 0 {
			entry = fmt.Sprintf("%d:\t%s\t%s-%s", subtask_id, src_name,
				subtask.Src.Ver, subtask.Src.Rel)
			if len(subtask.Commit) > 0 {
				entry += fmt.Sprintf("\t%s", subtask.Commit)
				}
		} else {
			entry = format.Preface
			for _, el := range format.Elements {
				switch el.Type {
					case 'd':
						entry += fmt.Sprintf("%v", datetime)
					case 't':
						entry += fmt.Sprintf("%d", task_id)
					case 's':
						entry += fmt.Sprintf("%d", subtask_id)
					case 'n':
						entry += fmt.Sprintf("%s", src_name)
					case 'v':
						entry += fmt.Sprintf("%s-%s", subtask.Src.Ver, subtask.Src.Rel)
					case 'h':
						entry += fmt.Sprintf("%s", subtask.Commit)
					default:
						err = fmt.Errorf("Invalid conversion %%v", el.Type)
						return
					}
				entry += el.Delim
				}
			}
		desc = append(desc, entry)
		}

	return
	}

//-----------------------------------------------------------------------------
// Print subtasks of a task with a given id using given format or a std one

func (a *Archive) PrintTask(task_id TaskId, format cfmt.CFormat) error {
	datetime, desc, err := a.getTaskDesc(task_id, format)
	if err != nil {
		return err
		}
	if format.Size() == 0 {
		fmt.Printf("Task #%d:\n\t/ %v /\n", task_id, datetime)
		for _, entry := range desc {
			fmt.Println("\t" + entry)
			}
	} else {
		for _, entry := range desc {
			fmt.Println(entry)
			}
		}
	return nil
	}
