/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package hlp

import (
	"encoding/csv"
	"io"
	"fmt"
	"os"
	"strings"
	"time"
	)

//-----------------------------------------------------------------------------
// Bottom-level functions

func replaceColons(s string) string {
	return strings.ReplaceAll(s, ":", " --")
	}

//-----------------------------------------------------------------------------
// Print a given special message

func printSpecialMsg(status, msg string) {
	var delim string
	if len(status) > 0 && len(msg) > 0 {
		delim = ": "
		}
	fmt.Println("[" + status + delim + replaceColons(msg) + "]")
	}

func PrintWarn(msg string) {
	printSpecialMsg("WARNING", msg)
	}

func PrintErr(msg string) {
	printSpecialMsg("ERROR", msg)
	}

//-----------------------------------------------------------------------------
// Convert date string to time.Time object

func ConvStrToTime(date string) (time.Time, error) {
	if date == "" {
		return time.Time{}, nil
		}
	datetime, err := time.Parse("2006-01-02", date)
	if err != nil {
		err = fmt.Errorf("Wrong date... %v", replaceColons(err.Error()))
		}
	return datetime, err
	}

//-----------------------------------------------------------------------------
// Check that a given dir exists

func CheckTargetDir(path string) error {
	f, err := os.Stat(path)
	if err != nil {
		return fmt.Errorf("Can't locate the target dir -- %s", err.Error())
		}
	if !(f.Mode().IsDir()) {
		return  fmt.Errorf("%q is not a dir", path)
		}
	return nil
	}

//-----------------------------------------------------------------------------
// Announce a given task and return a function that can be used to monitor a
// timming of that task (with a use of a closure that captures the 'start' var)

type Timer struct {
	Silent bool
	}

func (t *Timer) StartTask(msg string) func() {
	if t.Silent {
		return func() { return }
		}
	start := time.Now()
	if msg != "" {
		fmt.Println(msg + " ... ")
		}
	return func () {
		spec_msg := fmt.Sprintf("DONE in %.2fs", time.Since(start).Seconds())
		printSpecialMsg(spec_msg, "")
		}
	}

//-----------------------------------------------------------------------------
// Parse CSV multiline string

func ParseCSV(multiline string, values map[string]string) error {
	// Clearing the map first
	for k := range values {
		delete(values, k)
		}
	r := csv.NewReader(strings.NewReader(multiline))
	r.FieldsPerRecord = 2
	for {
		record, err := r.Read()
		if err == io.EOF {
			break
			}
		if err != nil {
			return err
			}
		values[strings.TrimSpace(record[0])] = strings.TrimSpace(record[1])
		}
	return nil
	}

//-----------------------------------------------------------------------------
// Construct CSV multiline string out of a given map

func ConstructCSV(values map[string]string) string {
	var multiline string
	for key, val := range values {
		multiline += fmt.Sprintf("%q,%q\n", key, val)
		}
	return multiline
	}

//-----------------------------------------------------------------------------
// Print a delimiter-line

func PrintDelimLine() {
	fmt.Println(strings.Repeat("=", 80))
	}
