/*
	alt-tasks : utility for observing ALT Linux tasks
	Copyright (C) 2020 Alexey Appolonov

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package main
import (
	"internal/cflag"
	"internal/hlp"
	"internal/home"

	"errors"
	"flag"
	"fmt"
	"os"
	"path/filepath"
	"runtime"
	"strings"
	)

const _APP_NAME = "alt-tasks"
const _SRC_LIST_FILENAME = "src_list"
const _SRC_LIST_SAMPLE   = "sisyphus, /ALT/repo/sisyphus/index/src"

var cpu_n = runtime.NumCPU()

//-----------------------------------------------------------------------------
// Command line arguments

var target   string
var src_name string
var task_id  uint
var update   bool
var notrace  bool
var verbose  bool

const _DATE_DESC = "Date in YYYY-MM-DD format"
const _FMT_DESC  = `Custom format of an output using following conversions:
	%d - date
	%t - task id
	%s - subtask id
	%n - name of a package
	%v - version (epoch:version-release) of a package
	%h - hash sum`
var after_date  = cflag.Date("after",  _DATE_DESC)
var before_date = cflag.Date("before", _DATE_DESC)
var format = cflag.Fmt("format", _FMT_DESC)

func init() {
	flag.StringVar(&target, "target", "", "[Required] Alias of a path " +
		"stated in \"" + _SRC_LIST_FILENAME + "\"")
	flag.StringVar(&src_name, "src_name", "", "Name of a source package")
	flag.UintVar(&task_id, "id", 0, "Task Id")
	flag.BoolVar(&update,  "update",  false, "Do not use existing binary dumps")
	flag.BoolVar(&notrace, "notrace", false, "Do not make binary dumps")
	flag.BoolVar(&verbose, "verbose", false, "Verbose output")
	}

//-----------------------------------------------------------------------------
// Terminate the program with a given error message

func fatalErr(err error) {
	hlp.PrintErr(err.Error())
	os.Exit(1)
	}

//-----------------------------------------------------------------------------
// Check given arguments

func checkArgs() error {

	err := ""
	got_range := !after_date.IsZero() || !before_date.IsZero()

	if target == "" {
		err = "'target' flag is required"
	} else if len(flag.Args()) > 0 {
		unflagged := strings.Join(flag.Args(), ", ")
		err = fmt.Sprintf("Unflaged arguments %q", unflagged)
	} else if task_id != 0 {
		if src_name != "" {
			err = "Can't use 'id' and 'src_name' simultaneously"
		} else if got_range {
			err = "Can't use 'id' flag with 'src_name'/'after'/'before' flags"
			}
	} else if src_name == "" && !got_range {
		err = "There is no search criteria"
	} else {
		full_range := !after_date.IsZero() && !before_date.IsZero()
		if full_range && after_date.After(*before_date) {
			err = "'after' date should be lesser than 'before' date"
			}
		}

	if err != "" {
		return errors.New(err)
		}
	return nil
	}

//-----------------------------------------------------------------------------

// Recursively search for and parse all "d-t-s-evr.list" files in a given dir,
// form a task archive and print those tasks that satisfy given conditions

func main() {

	// Parsing given arguments (printing help if no arguments were passed)
	flag.Parse()
	if len(os.Args) == 1 {
		flag.Usage()
		os.Exit(2)
		}

	// Determining a file that holds a map of src targets
	list_file, err := home.NewListFile(_SRC_LIST_FILENAME, _SRC_LIST_SAMPLE)
	if err != nil {
		fatalErr(err)
		}

	// Recreating a home dir
	app_home, err := home.PrepareHome(_APP_NAME, []home.ListFile{*list_file,})
	if err != nil {
		fatalErr(err)
		}

	// Checking the values of a given arguments
	if err := checkArgs(); err != nil {
		fatalErr(err)
		}

	// Creating a worker obj that will form and handle an archive of tasks
	w := NewWorker(cpu_n)

	// Creating a Timer obj
	timer := hlp.Timer{!verbose}

	// Using a binary dump if there is one
	dump_path, dump_exists := app_home.DumpPath(target)
	if !update && dump_exists {
		msg := fmt.Sprintf("Decoding an archive from %q", dump_path)
		task_timing := timer.StartTask(msg)
		if err := w.loadDump(dump_path); err != nil {
			hlp.PrintErr(err.Error())
		} else {
			task_timing()
			}
		}

	// Flag that shows either a new scanning was performed or not
	new_scan := false

	// Scanning the target dir if binary dump had no use
	if w.archive.Empty() {

		// Getting a src dir path
		path, err := app_home.GetVal(_SRC_LIST_FILENAME, target)
		if err != nil {
			fatalErr(err)
			}

		// Receiving task records and writing them to the archive in a backgrnd
		w.writing_wg.Add(1)
		go w.writeDocuments()

		// Scanning a chosen source dir
		task_timing := timer.StartTask(fmt.Sprintf("Scanning %q", path))
		if err := hlp.CheckTargetDir(path); err != nil {
			fatalErr(err)
			}
		if err := filepath.Walk(path, w.handleFile); err != nil {
			fatalErr(err)
			}

		// Waiting for parsers to finish their jobs and then closing the channel
		w.parsing_wg.Wait()
		close(w.documents)
		w.writing_wg.Wait()

		task_timing()
		new_scan = true
		}

	msg := "Selecting tasks that meet the given criteria"
	task_timing := timer.StartTask(msg)
	if err := w.showMatchingTasks(task_id, src_name, *after_date, *before_date,
			*format, verbose); err != nil {
		fatalErr(err)
		}
	task_timing()

	// Terminating if there is nothing to save or binary dump is unwanted
	if !new_scan || notrace {
		os.Exit(0)
		}

	msg = fmt.Sprintf("Making a binary dump into %q", dump_path)
	task_timing = timer.StartTask(msg)
	if err := w.makeDump(dump_path); err != nil {
		fatalErr(err)
		}
	task_timing()
	}
